use serde::Deserialize;
use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::to_wasm::ToWasm;
use super::AdditionalProperties;
use crate::compare::ElispFact;
use crate::types::FixedWidthArea;
use crate::types::GetAffiliatedKeywords;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize, Deserialize)]
pub struct WasmFixedWidthArea {
    #[serde(flatten)]
    pub(crate) additional_properties: AdditionalProperties,
    pub(crate) value: String,
}

to_wasm!(
    WasmFixedWidthArea,
    FixedWidthArea<'s>,
    original,
    wasm_context,
    { WasmAstNode::FixedWidthArea(original) },
    { "fixed-width".into() },
    {
        let additional_properties = original
            .get_affiliated_keywords()
            .to_wasm(wasm_context.clone())?;

        let value = original.get_value();

        Ok((
            Vec::new(),
            WasmFixedWidthArea {
                additional_properties,
                value,
            },
        ))
    }
);
